(use-translation _ "alterator-x11")

(include-as-is "/dialog/functions")

(define cardslist (woo-read-names "/video_cards/auto_detect" (woo-list "/video_cards/auto_detect")))
(define carddata (woo-read (string-append "/video_cards/auto_detect/" (car cardslist))))

(define card (woo-get-option (woo-first-command carddata) 'name))

(define autodetect (woo-first-command (woo-read "/monitors/auto_detect")))
(define autodetect_video (woo-first-command (woo-read "/xconf/autodetect")))
(define selected-data (woo-first-command (woo-read "/xconf/data")))
(define inittab (woo-first-command (woo-read "/inittab/inittab")))

(define firsttime (woo-get-option (woo-first-command (woo-read "/inittab/firsttime")) 'firsttime))

(define runlevel (if (string=? firsttime "true") "5" (woo-get-option inittab 'runlevel)))
(define monitor	(woo-get-option autodetect 'name))
(define recommended_resolution (woo-get-option autodetect 'recommended))
(define recommended_video (woo-get-option autodetect_video 'card))
(define recommended_depth (woo-get-option autodetect_video 'depth))
(define ddcresolutions (if (empty-string? (woo-get-option autodetect 'detected)) '() (string-split  (woo-get-option autodetect 'detected) #\ )))
(define selected-driver (woo-get-option selected-data 'card))
(define selected-monitor (woo-get-option selected-data 'monitor))
(define selected-resolution (woo-get-option selected-data 'resolution))
(define selected-color (woo-get-option selected-data 'color))

;return selected driver or autodetected data
(define (get-driver)
  (if (empty-string? selected-driver)
      (if (null? recommended_video) "" recommended_video)
      selected-driver))

;return detected video card
(define (get-card)
  (if (null? card) "" card))

;return selected monitor or autodetected data
(define (get-monitor)
  (if (empty-string? selected-monitor) monitor selected-monitor))

(define (string-list-index str searchlist) (or (list-index (lambda (x) (string=? str x)) searchlist) #f))
(define (num-list-index2 num searchlist) (or (list-index (lambda (x) (= num (cdr x))) searchlist) 0))


;color names
(define colors  (list (cons (_ "256 colors (8 bit)") 8)
		      (cons (_ "32 thousand colors (15 bits)") 15)
		      (cons (_ "65 thousand colors (16 bits)") 16)
		      (cons (_ "16 million colors (24 bits)") 24)
		      (cons (_ "4 billion colors (32 bits)") 32)))

;resolutions
(define vesa-resolutions '("640x480"
			   "800x600"
			   "1024x768"
			   "1280x1024"
			   "1600x1200"
			   "1920x1080"
			   "1920x1200"
			   "2048x1536"))

(define non-standard-resolutions '("720x576"
				   "832x624"
				   "848x480"
				   "960x720"
				   "1024x600"
				   "1024x736"
				   "1120x840"
				   "1152x864"
				   "1152x870"
				   "1280x720"
				   "1280x768"
				   "1280x800"
				   "1280x854"
				   "1280x960"
				   "1400x1050"
				   "1440x900"
				   "1600x1024"
				   "1600x900"
				   "1680x1050"))
;return ddcresultions if it detected
(define resolutions (if (null? ddcresolutions) 
			(append vesa-resolutions non-standard-resolutions) 
			ddcresolutions))

(on-apply (test-and-apply))

;stay on this page until x11 don't work
;(on-error '(/ctrl/look action ignore))

(define (get-depth)
  (cdr (list-ref colors (depth current))))

(define (depth2string x)
  (car (list-ref colors (num-list-index2 (string->number x) colors))))

(define (get-resolutions resolution)
  (let* ((max-res (string-split resolution #\x))
	 (maxx (string->number (car max-res)))
	 (maxy (string->number (cadr max-res))))
    (string-join
     (map (lambda (e)
	    (string-append (number->string (car e))
			   "x"
			   (number->string (cdr e))))
	  (sort
	   (delq #t
		 (map (lambda (e)
			(let* ((res (string-split e #\x))
			       (x (string->number (car res)))
			       (y (string->number (cadr res))))
			  (or (or (> x maxx)
				  (and (= x maxx) (> y maxy)))
			      (cons x y))))
		      (append vesa-resolutions non-standard-resolutions)))
	   (lambda (e x)
	     (or (< (car e) (car x))
		 (and (= (car e) (car x))
		      (< (cdr e) (cdr x))))))) ",")))

(define (get-resolution)
  (if (string=? (resolution text) "auto") "auto"
      (get-resolutions (resolution text))))

;made temporary X11 config file, fill with data,run test and commit
(define (test-and-apply)
  (woo-catch
   (lambda()
     (woo-write "/xconf/data" 'monitor (get-monitor))
     (woo-write "/xconf/data" 'card (get-driver))
     (write (get-depth))
     (woo-write "/xconf/data" 'color-depth (get-depth))
     (woo-write "/xconf/data" 'resolution (get-resolution))
     (if (string=? firsttime "true") (woo-write "/xconf/data" 'kbd "init"))
     (woo-write "/inittab/inittab" 'runlevel 
		(if (startx11 checked) "5" "3"))
     (if (zero? (geteuid)) 
	 (woo-new "/xconf/test" ) 
	 (run-dialog `("/information" message ,(string-append "<b>Configuration file saved to</b> \n" (getenv "HOME") "/xorg.conf"))))
     (woo-write "/xconf/commit" ) #t)
   (lambda (x) (run-dialog `("/yesno" message ,"Error occured during X11 test, ignore?")))))

; save data if url changed
(define (change url)
  (woo-write "/xconf/data" 'card (get-driver))
  (woo-write "/xconf/data" 'monitor (get-monitor))
  (goto url
	'saved_depth (depth current)
	'x11test (global 'x11test)
	'selected_driver (get-driver)
	'selected_monitor (get-monitor)
	'self	(global 'self)
	'saved_resolution (resolution current)))

; move recommended values into selected values
(define (apply-recommended) 
  (or (empty-string? recommended_depth) (begin
				   (depth current (num-list-index2 (string->number recommended_depth) colors))))
  (or (empty-string? recommended_video) (begin
				   (set! selected-driver recommended_video) 
				   (drv text recommended_video)))
  (or (empty-string? recommended_resolution) (begin
                                  (resolution current (+ 1 (string-list-index recommended_resolution resolutions)))))
  (or (empty-string? monitor) (begin
			 (set! selected-monitor monitor) 
			 (mon text monitor)))
  (woo-write "/xconf/data" 'card (get-driver))
  (woo-write "/xconf/data" 'monitor (get-monitor)))

; 
(vbox spacing 1
    (groupbox (_ "Detected hardware :")
    	    sizepolicy '(expanding . fixed)
	    (vbox spacing 1
		  (or (empty-string? (get-card) ) (hbox spacing 0 (label (_ "Video card:")) (label (get-card))))
		  (or (empty-string? monitor) 
		      (hbox spacing 0 (label (_ "Monitor:")) (label monitor)))))
      (groupbox (_ "Recomended settings :")
      	    sizepolicy '(expanding . fixed)
	    (vbox spacing 1
		  (or (empty-string? recommended_video)  
		      (hbox spacing 0 (label (_ "Autodetected driver for video card:")) (label recommended_video )))
		  (or (empty-string? recommended_depth)  
		      (hbox spacing 0 (label (_ "Autodetected depth for video driver:")) (label (depth2string recommended_depth) )))
		  (or (empty-string? recommended_resolution) 
		      (hbox spacing 0 (label (_ "Best autodetected resolution for monitor:")) (label recommended_resolution)))
		  (hbox (button (_ "Apply autodetected") sizepolicy 'fixed (on-click (apply-recommended))))))
      (groupbox (_ "Current settings :") 
      	    sizepolicy '(expanding . fixed)
      (vbox spacing 1
      (hbox spacing 1
	    (label (_ "Driver:") (relative-width 15 page))
	    (id 'drv (label (get-driver) (relative-width 20 page)))
	    (button (_ "Change...") (on-click (change "/x11/change_card"))))
      (hbox spacing 1
	    (label (_ "Monitor:") (relative-width 15 page))
	    (id 'mon (label (get-monitor) (relative-width 20 page)))
	    (button (_ "Change...") (on-click (change "/x11/change_monitor"))))
      (hbox spacing 1
	    (label (_ "Color depth:") (relative-width 33 page))
	    (id 'depth (combobox (items colors) (relative-width 30 page)
				 current (if (global 'saved_depth) (global 'saved_depth) 
					     (if (empty-string? selected-color) 
						 (num-list-index2 (string->number 
								   (if (empty-string? recommended_depth) 
								       "24" 
								       recommended_depth)) 
								  colors) 
						 (num-list-index2 (string->number selected-color) colors))))))
      (hbox spacing 1
	    (label (_ "Resolution:") (relative-width 33 page))
	    (id 'resolution (combobox (items (append `("auto") resolutions)) (relative-width 30 page)
				      current (if (global 'saved_resolution) 
						  (global 'saved_resolution)
						  (+ 1 
						     (or (string-list-index (if (empty-string? selected-resolution) 
										recommended_resolution 
										selected-resolution)
									    resolutions )
							 -1))))))
      (hbox spacing 1
	    (label (_ "Start X11 on system booting") (relative-width 33 page))
	    (id 'startx11 (checkbox ""  checked (string=? runlevel "5") (relative-width 30 page) enabled (zero? (geteuid)))))))
      (if (not (global 'x11test)) 
      (groupbox ""
      		sizepolicy '(expanding . fixed)
	  (hbox (button (_ "Test and save") (on-click (test-and-apply)))))))
